/* Copyright (c) 2003 ActiveState, a Division of Sophos
   See the file LICENSE.txt for licensing information. */

/* Support for the Open/Find toolbar
 *  - this toolbar is (for now) showable or not as a whole.  There
 *    is no way to show only the Open or only the Find items.
 *  - Hitting escape in any text field focuses back on the current
 *    editor if there is one.
 *  - There is autocompletion in the Open text field based on the
 *    current working directory for the current file.
 *  - the Find text fields have preference-backed MRUs.
 *  - The find capabilities share 'sessions' 
 */


//---- globals

var _findtoolbarlog;
var gFindTextboxContext;
var gFindFilesTextboxContext;
var _find_relativeFnameExprRE = /^[\.|\w][^:]/;
var _find_absoluteFnameExprRE = /^\w:|^\/|^\\\\|^\/\//;
var gFindToolbar_FindInFiles_Textbox;

//--- onload handler.
// This creates the autocomplete session and initializes the
// 
function findtoolbar_onload()
{
    try {
        _findtoolbarlog = getLoggingMgr().getLogger('findtoolbar');
        //_findtoolbarlog.setLevel(LOG_DEBUG);
        var textbox_open = document.getElementById("textbox_open");
        var session = new Findtoolbar_OpenAutoCompleteSession(textbox_open);
        textbox_open.addSession(session);
        gFindTextboxContext = Components.classes["@activestate.com/koFindContext;1"]
                    .createInstance(Components.interfaces.koIFindContext);
        gFindTextboxContext.type = Components.interfaces.koIFindContext.FCT_CURRENT_DOC;
    
        gFindFilesTextboxContext = Components.classes[
            "@activestate.com/koFindInFilesContext;1"]
            .createInstance(Components.interfaces.koIFindInFilesContext);
        gFindFilesTextboxContext.type = Components.interfaces.koIFindContext.FCT_IN_FILES;
        gFindFilesTextboxContext.cwd = window.gFindInFilesCwd;
    
        var textbox_find = document.getElementById('textbox_find');
        var session = new MRUAutoCompleteSession(textbox_find);
        textbox_find.setAttribute("_session", session)
    
        gFindToolbar_FindInFiles_Textbox = document.getElementById('textbox_findInFiles');
        var session = new MRUAutoCompleteSession(gFindToolbar_FindInFiles_Textbox);
        gFindToolbar_FindInFiles_Textbox.setAttribute("_session", session)
    } catch (e) {
        log.exception(e); // not findtoolbar in case that's what failed.
    }
}

function _Findtoolbar_OpenTextboxJumpTo(field, value) {
}

function Findtoolbar_OpenTextboxKeyPress(field, event) {
    try {
        switch (event.keyCode) {
            case event.DOM_VK_RETURN:
                var value = field.value;
                var cwd = View_GetCwd();
                if (! _find_absoluteFnameExprRE.test(value)) {
                    value = os.path.join(cwd, value);
                }
                if (event.ctrlKey) {
                    var dirname, filename;
                    if (os.path.isdir(value)) {
                        dirname = os.path.abspath(value);
                        filename = '';
                    } else {
                        dirname = os.path.abspath(os.path.dirname(value));
                        filename = os.path.basename(value);
                    }
                    var paths = filepicker_openFiles(dirname, filename);
                    if (paths) {
                        open_openMultipleURIs(paths, 'editor');
                    }
                } else {
                    if (! field.value) return;
                    if (os.path.isdir(value)) {
                        // Don't do anything w/ directories, just get ready
                        // to continue
                        field.selectionStart = field.value.length;
                        field.selectionEnd = field.value.length;
                        return;
                    }
                    field.value = '';
                    // It's got to be a filename.  It may be a relative
                    // path, in which case make it absolute
                    if (! _find_absoluteFnameExprRE.test(value)) {
                        value = os.path.abspath(os.path.join(cwd, value));
                    }
                    // It may be a glob -- if yes, then expand it.
                    if (value.indexOf('*') != -1 ||
                        value.indexOf('?') != -1) {
                        var glob = Components.classes["@activestate.com/koGlob;1"].
                               getService(Components.interfaces.koIGlob);
                        var paths = new Object();
                        paths = glob.glob(value, new Object())
                        for (var i = 0; i < paths.length; i++) {
                            open_openURI(paths[i]);
                        }
                    } else {
                        open_openURI(value);
                    }
                    _Findtoolbar_OpenTextboxJumpTo(field, value);
                }
                event.stopPropagation();
                event.preventDefault();
                event.cancelBubble = true;
                break;
            case event.DOM_VK_TAB:
                // I'm pretty sure this isn't a documented attribute,
                // but 'open' doesn't work as I expect it to.
                if (field.resultsPopup.hasAttribute('menutobedisplayed')) {
                    field.selectionStart = field.value.length;
                    field.selectionEnd = field.value.length;
                } else {
                    if (event.shiftKey) {
                        gFindToolbar_FindInFiles_Textbox.focus();
                    } else {
                        document.getElementById('textbox_find').focus();
                    }
                }
                event.preventDefault();
                event.stopPropagation();
                break;
            case event.DOM_VK_ESCAPE:
                Findtoolbar_blur(field);
                event.stopPropagation();
                break;
        }
    } catch (e) {
        log.exception(e);
    }
}

function Findtoolbar_blur(field) {
    field.value = '';
    gViewMgr.currentView.setFocus();
}


function Findtoolbar_FindTextboxKeypress(field, event) {
    try {
        if (event.type != 'keypress') {
            return;
        }
        if (String.fromCharCode(event.charCode) == 'a') {
            if (event.altKey && ! event.shiftKey && ! event.ctrlKey) {
                Find_FindAll(window, gFindTextboxContext, field.value);
                event.stopPropagation();
                event.preventDefault();
                return;
            }
        }
        switch (event.keyCode) {
            case event.DOM_VK_TAB:
                    if (event.shiftKey) {
                        document.getElementById('textbox_open').focus();
                    } else {
                        gFindToolbar_FindInFiles_Textbox.focus();
                    }
                    event.stopPropagation();
                    event.preventDefault();
                break;
            case event.DOM_VK_RETURN:
                if (! field.value || gViewMgr.currentView.getAttribute('type') != 'editor') {
                    return;
                }
                MRU_updateFromWidget(field);
                if (event.ctrlKey) {
                    launch_Find(field.value);
                } else {
                    MRU_updateFromWidget(field);
                    if (event.shiftKey) {
                        findSvc.options.searchBackward = !findSvc.options.searchBackward;
                        Find_FindNext(window, gFindTextboxContext, field.value,
                                      "find", false, false);
                        findSvc.options.searchBackward = !findSvc.options.searchBackward;
                    } else {
                        Find_FindNext(window, gFindTextboxContext, field.value,
                                      "find", false, false);
                    }
                }
                event.stopPropagation();
                break;
            case event.DOM_VK_ESCAPE:
                gViewMgr.currentView.setFocus();
                event.stopPropagation();
                break;
        }
    } catch (e) {
        log.exception(e);
    }
}

function Findtoolbar_FindFilesKeyPress(field, event) {
    try {
        if (event.type != 'keypress') return;
        switch (event.keyCode) {
            case event.DOM_VK_TAB:
                if (event.shiftKey) {
                    document.getElementById('textbox_find').focus();
                } else {
                    document.getElementById('textbox_open').focus();
                }
                event.stopPropagation();
                event.preventDefault();
                break;
            case event.DOM_VK_RETURN:
                var findTerm = document.getElementById('textbox_find').value;
                if (!findTerm) return;
                if (!field.value) return;
                var findSvc = Components.classes["@activestate.com/koFindService;1"].
                       getService(Components.interfaces.koIFindService);
                var value = field.value;
                var view = gViewMgr.currentView;
                var quantifierFound = ((value.indexOf('*') != -1) ||
                                       value.indexOf('?') != -1);
                var lastSlash = Math.max(value.lastIndexOf('/'),
                                         value.lastIndexOf('\\')) + 1;
                // Split out the expression into a directory and a
                // file pattern.
                if (lastSlash == -1) {
                    lastSlash = value.length;
                }
                // We want to map the following:
                // value               dirname    filetypes
                // 'foo/bar'           'foo/bar'  ''
                // 'foo/*.py'          'foo'      '*.py'
                // '../foo/bar/*.py'   '../foo'   '*.py'
                // 'foo'               'foo'      ''
                if (quantifierFound) {
                    filetypes = value.slice(lastSlash);
                    dirname = value.slice(0, lastSlash-1);
                } else {
                    filetypes = '';
                    dirname = value;
                }
                findSvc.options.encodedIncludeFiletypes = filetypes;
                findSvc.options.encodedFolders = dirname;
                if (event.ctrlKey) {
                    launch_FindInFiles(findTerm, dirname, filetypes);
                } else {
                    MRU_updateFromWidget(field);
                    gFindFilesTextboxContext.cwd = View_GetCwd();
                    Find_FindAllInFiles(window,
                                        gFindFilesTextboxContext,
                                        findTerm,
                                        findTerm);
                }
                event.stopPropagation();
                break;
            case event.DOM_VK_ESCAPE:
                gViewMgr.currentView.setFocus();
                event.stopPropagation();
                break;
        }
    } catch (e) {
        log.exception(e);
    }
}

function Findtoolbar_GotoOpenTextbox() {
    document.getElementById('textbox_open').focus();
}

function Findtoolbar_ResetFindTextbox(elt) {
    if (elt.session)
        elt.session.clearCache();
}

function Findtoolbar_GotoFindTextbox() {
    document.getElementById('textbox_find').focus();
}

function Findtoolbar_OpenAutoCompleteSession(widget)
{
    try {
        // This is put inside "if (widget)" because to subclass one needs to run
        // "new Findtoolbar_OpenAutoCompleteSession();" and then later call this same function
        // with the needed arguments. Thanks a lot JavaScript.
        if (widget) {
            // Hack ourselved into the autocomplete session for the widget
            _findtoolbarlog.info("Findtoolbar_OpenAutoCompleteSession(widget) init");
            var sessionName = "findtoolbar";
            //widget.mSessions[sessionName] = this; // this creates another session!
            widget.mListeners[sessionName] = new (widget.mAutoCompleteListener)(sessionName);
            ++widget.sessionCount;
            // Do not use <Tab> to cycle through autocomplete list.
            // widget.useTabComplete = false;
        }
        this.clearCache();
    } catch (e) {
        log.exception(e);
    }
}

Findtoolbar_OpenAutoCompleteSession.prototype = new Object();
Findtoolbar_OpenAutoCompleteSession.prototype.constructor = Findtoolbar_OpenAutoCompleteSession;

Findtoolbar_OpenAutoCompleteSession.prototype.clearCache = function() {
    this._dirCache = {}
}

Findtoolbar_OpenAutoCompleteSession.prototype.lookupFromValue = function(value) {
    if (value in this._dirCache) {
        return this._dirCache[value];
    }
    var files;
    if (! os.path.isdir(value)) {
        files = null;
    } else {
        try {
            files = os.listdir(value, new Object, files);
        } catch(e) {
            // exception when autocompleting unc file paths that are
            // incomplete or the actual share.  for example,
            // \\americano\home\ does not complete, but
            // \\americano\home\shanec\ does.
        }
    }
    this._dirCache[value] = files;
    return files;
}

Findtoolbar_OpenAutoCompleteSession.prototype.onStartLookup = function(value, prevResult, listener) {
    try {
        var textbox_open = document.getElementById("textbox_open");
        // I'm not sure why this is called twice.  The second call seems
        // to always have a null 'prevResult', and should be ignored.
        //if (typeof(prevResult) != 'undefined' && prevResult == null) return;
    
        var lastSlash = Math.max(value.lastIndexOf('/'),
                                 value.lastIndexOf('\\'));
        var abspathspecified = false;
        var relpathspecified = false;
        var nopathspecified = false;
        if (! _find_absoluteFnameExprRE.test(value) || lastSlash == -1) {
            // It's a relative path
            // 1) find out what the directory is
            // 2) get the listing from that directory
            // the relative directory is up to the last slash
            var relDirName;
            var cwd = View_GetCwd();
            if (lastSlash == -1) {
                nopathspecified = true;
                relDirName = '';
                absDirName = cwd;
            } else {
                relpathspecified = true;
                relDirName = value.slice(0, lastSlash+1);
                absDirName = os.path.join(cwd, relDirName);
            }
        } else {
            abspathspecified = true;
            absDirName = value.slice(0, lastSlash+1);
        }
        files = this.lookupFromValue(absDirName);
        if (!files) return; // possibly because not a dir
        var relfiles = files.slice();
        var i;
        if (relpathspecified) {
            for (i = 0; i < relfiles.length; i++) {
                relfiles[i] = os.path.join(relDirName, relfiles[i]);
            }
        } else if (abspathspecified) {
            for (i = 0; i < relfiles.length; i++) {
                relfiles[i] = os.path.join(absDirName, relfiles[i]);
            }
        } else if (nopathspecified) {
            // Add all of the current view names ??
        }
        var results = Components.classes["@mozilla.org/autocomplete/results;1"]
                      .createInstance();
        results = results.QueryInterface(Components.interfaces.nsIAutoCompleteResults);
        results.searchString = value;
        // no default, i.e. hitting <Return> will use the current string rather
        // than the first match in the Modeline
        results.defaultItemIndex = -1;
        results.param = null;

        var item, filename;
        var validEntries = 0;
        for (i = 0; i < relfiles.length; i++) {
            filename = relfiles[i];
            if (filename.indexOf(value) != -1) {
                // not sure this is ideal, but trying it.
                // I'm thinking that it might be nice to be able to enter
                // ../editor/.py and have the .py files show up
                item = Components.classes["@mozilla.org/autocomplete/item;1"]
                           .createInstance();
                item = item.QueryInterface(Components.interfaces.nsIAutoCompleteItem);
                item.value = filename;
                results.items.AppendElement(item);
                validEntries++;
            }
        }
        if (validEntries == 0) {
            listener.onAutoComplete(results,
                Components.interfaces.nsIAutoCompleteStatus.noMatch);
            return;
        }
        
        var numEntries = String(Math.min(20, validEntries));
        document.getElementById("textbox_open").setAttribute('maxrows',
                                                          numEntries);
        listener.onAutoComplete(results,
            Components.interfaces.nsIAutoCompleteStatus.matchFound);
    } catch (e) {
        _findtoolbarlog.exception(e);
    }
}

Findtoolbar_OpenAutoCompleteSession.prototype.onStopLookup = function()
{
    _findtoolbarlog.debug("onStopLookup called\n");
}

Findtoolbar_OpenAutoCompleteSession.prototype.onAutoComplete = function(value, prevResult, listener)
{
    _findtoolbarlog.debug("Search Session: onAutoComplete, value =" + value + "\n");
    if (value) {
        Findtoolbar_FindTextboxJumpTo(document.getElementById('textbox_open'), value);
    }
}

Findtoolbar_AddBrowsedDirectory = function() {
    _findtoolbarlog.debug("Findtoolbar_AddBrowsedDirectory()");
    try {
        var doReplace = false; // are we replacing the entire field or inserting into it.
        var curDir;
        var os = Components.classes["@activestate.com/koOs;1"].getService();
        var textbox = gFindToolbar_FindInFiles_Textbox;
        var encodedFolders = textbox.value;
        if (textbox.selectionStart == 0 &&
            textbox.selectionEnd == encodedFolders.length) {
            doReplace = true;
            curDir = os.path.realpath(encodedFolders.split(os.pathsep)[0]);
        } else {
            if (textbox.selectionStart == textbox.selectionEnd) {
                curDir = os.path.realpath(encodedFolders.split(os.pathsep)[0]);
            } else {
                curDir = os.path.realpath(encodedFolders.slice(textbox.selectionStart,
                                                                   textbox.selectionEnd).split(os.pathsep)[0]);
            }
        }
        var newDir = filepicker_getFolder(curDir,
                                          "Choose a folder to add to the search list:");
        if (newDir) {
            if (doReplace) {
                textbox.value = newDir;
            } else {
                var oldValue = textbox.value;
                var selStart = textbox.selectionStart;
                var selEnd = textbox.selectionEnd;
                var newFolder = os.pathsep + newDir;
                var newValue = oldValue.slice(0, selStart) + newFolder
                               + oldValue.slice(selEnd, oldValue.length);
                textbox.value = newValue
                textbox.selectionStart = selStart;
                textbox.selectionEnd = selStart + newFolder.length;
            }
        }
    } catch(ex) {
        _findtoolbarlog.exception(ex);
    }
}
