from xpcom import components, ServerException

from koLanguageServiceBase import *

class koCPPLanguage(KoLanguageBase):
    name = "C++"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{0613C3CC-EAA4-47f9-B2C3-A73270FFFB19}"

    defaultExtension = ".cpp"
    commentDelimiterInfo = {
        "line": [ "//" ],
        "block": [ ("/*", "*/") ],
        "markup": "*",
    }
    _dedenting_statements = [u'throw', u'return', u'break', u'continue']
    _indenting_statements = [u'case', u'protected', u'private', u'public']
    _block_comment_styles = [sci_constants.SCE_C_COMMENT,
                            sci_constants.SCE_C_COMMENTDOC,
                            sci_constants.SCE_C_COMMENTDOCKEYWORD,
                            sci_constants.SCE_C_COMMENTDOCKEYWORDERROR]
    supportsSmartIndent = "brace"
    sample = """#include <windows.h>
#include <iostream>
#include <algorithm>
#include <vector>
#include <cstdlib>

using namespace std;    // This is a comment 
int main(int argc, char *argv[]) 
{
  if (argc != 2) {
    std::cerr << \"Usage:\\tsieve [iterations]"; 
    return 1;
  };

  size_t NUM = atoi(argv[1]);
  DWORD dw = ::GetTickCount();
  vector<char> primes(8192 + 1);
  vector<char>::iterator pbegin = primes.begin();
  vector<char>::iterator begin = pbegin + 2;
  vector<char>::iterator end = primes.end();

  while (NUM--) {
    fill(begin, end, 1);
    for (vector<char>::iterator i = begin; 
              i < end; ++i) {
      if (*i)  {
        const size_t p = i - pbegin;
        for (vector<char>::iterator k = i + p; 
          k < end; k += p) {
          *k = 0;
        }
      }
    }
  }
  DWORD dw2 = ::GetTickCount();
  std::cout << \"Milliseconds = \" << dw2-dw << std::endl;
  return 0;
}
"""

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_CPP)
            self._lexer.setKeywords(0, self._keywords)
            self._lexer.supportsFolding = 1
        return self._lexer

    # Taken from "The C++ Programming Language" 3rd Edition (published in 1997)
    
    _keywords = ["and", "and_eq",  "asm", "auto", "bitand", "bitor", "bool", "break",
                 "case", "catch", "char", "class", "compl", "const", "const_cast",
                 "continue", "default", "delete", "do", "double", "else", "enum",
                 "explicit", "export", "extern", "false", "float", "for", "friend", "goto",
                 "if", "inline", "int", "long", "mutable", "new", "not", "not_eq", "operator",
                 "or", "or_eq", "private", "protected", "public", "register",
                 "reinterpret_cast", "return", "short", "signed", "sizeof", "static",
                 "static_cast", "struct", "switch", "template", "this", "throw", "true",
                 "try", "typedef", "union", "unsigned", "using", "virtual", "void", "volatile",
                 "wchar_t", "while", "xor", "xor_eq"]

