from xpcom import components, ServerException

from koLanguageServiceBase import *

class koSQLLanguage(KoLanguageBase):
    name = "SQL"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{EC1B0777-D982-41af-906F-34923D602B72}"

    defaultExtension = ".sql"
    commentDelimiterInfo = {
        "line": [ "--" ],
        "block": [ ("/*", "*/") ],
    }
    _block_comment_styles = [sci_constants.SCE_C_COMMENT]
    
    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_SQL)
            self._lexer.setKeywords(0, self._keywords)
            self._lexer.supportsFolding = 0
        return self._lexer

    # LexSQL seems to ignore keywords completely
    _keywords = ["ADD", "ALL", "ALTER", "AND", "ANY", "AS",
                 "ASC", "AUTHORIZATION", "AVG", "BACKUP", "BEGIN",
                 "BETWEEN", "BINARY", "BREAK", "BROWSE", "BULK", "BY",
                 "CALL", "CASCADE", "CASE", "CAST", "CHAR",
                 "CHAR_CONVERT", "CHARACTER",  "CHECK", "CHECKPOINT",
                 "CLOSE", "CLUSTERED", "COALESCE", "COLUMN",
                 "COMMENT", "COMMIT", "COMMITTED", "CONNECT",
                 "COMPUTE", "CONFIRM",
                 "CONSTRAINT", "CONTAINS", "CONTAINSTABLE",
                 "CONTINUE", "CONTROLROW", "CONVERT", "COUNT",
                 "CREATE", "CROSS", "CURRENT", "CURRENT_DATE",
                 "CURRENT_TIME", "CURRENT_TIMESTAMP",
                 "CURRENT_USER", "CURSOR", "DATABASE", "DATE",
                 "DBA", "DBCC", "DBSPACE", "DEALLOCATE", "DEC",
                 "DECIMAL", "DECLARE",
                 "DEFAULT", "DELETE", "DENY", "DESC", "DISK", "DISTINCT",
                 "DISTRIBUTED", "DO", "DOUBLE", "DROP", "DUMMY",
                 "DUMP", "ELSE", "ELSEIF", "ENCRYPTED", "END",
                 "ENDIF", "ERRLVL", "ERROREXIT",
                 "ESCAPE", "EXCEPT", "EXCEPTION", "EXEC", "EXECUTE",
                 "EXISTS", "EXIT", "FETCH", "FILE", "FILLFACTOR", "FIRST",
                 "FLOAT", "FLOPPY", "FOR", "FOREIGN", "FREETEXT",
                 "FREETEXTTABLE", "FROM", "FULL", "GOTO",
                 "GRANT", "GROUP", "HAVING", "HOLDLOCK", "IDENTIFIED",
                 "IDENTITY", "IDENTITY_INSERT", "IDENTITYCOL",
                 "IF", "IN", "INDEX", "INNER", "INOUT", "INSERT",
                 "INSTEAD", "INT", "INTEGER", "INTERSECT", "INTO", "IS",
                 "ISOLATION",
                 "JOIN", "KEY", "KILL", "LEFT", "LEVEL", "LIKE",
                 "LINENO", "LOAD", "LOCK", "LONG", "MATCH", "MAX",
                 "MEMBERSHIP", "MESSAGE", "MIN", "MIRROREXIT",
                 "MODE", "MODIFY", "NAMED", "NATIONAL", "NATURAL",
                 "NOCHECK", "NOHOLDLOCK", "NONCLUSTERED",
                 "NOT", "NULL", "NULLIF", "NUMERIC", "OF", "OFF", "OFFSETS",
                 "ON", "ONCE", "ONLY", "OPEN",
                 "OPENDATASOURCE", "OPENQUERY",
                 "OPENROWSET", "OPTION", "OPTIONS", "OR", "ORDER",
                 "OTHERS", "OUT", "OUTER", "OVER", "PASSTHROUGH",
                 "PERCENT", "PERM",
                 "PERMANENT", "PIPE", "PRECISION", "PREPARE",
                 "PRIMARY", "PRINT", "PRIVILIGES", "PROC",
                 "PROCEDURE", "PROCESSEXIT", "PUBLIC",
                 "RAISERROR", "READ", "READTEXT", "REAL", "RECONFIGURE",
                 "REFERENCE", "REFERENCES", "RELEASE", "REMOTE",
                 "RENAME", "REPEATABLE", "REPLICATION",
                 "RESOURCE", "RESTORE", "RESTRICT", "RETURN", "REVOKE",
                 "RIGHT", "ROLLBACK", "ROWCOUNT",
                 "ROWGUIDCOL", "RULE", "SAVE", "SAVEPOINT", "SCHEDULE",
                 "SCHEMA",
                 "SELECT", "SERIALIZABLE", "SESSION_USER",
                 "SET", "SETUSER", "SHARE", "SHUTDOWN", "SMALLINT",
                 "SOME", "SQLCODE", "SQLSTATE", "START",
                 "STATISTICS", "STOP", "SUBTRANS", "SUBTRANSACTION",
                 "SUM", "SYNCHRONIZE", "SYNTAX_ERROR", "SYSTEM_USER",
                 "TABLE", "TAPE", "TEMP", "TEMPORARY",
                 "TEXTSIZE", "THEN", "TIME", "TINYINT", "TO", "TOP", "TRAN",
                 "TRANSACTION", "TRIGGER", "TRUNCATE",
                 "TSEQUAL", "UNCOMMITTED", "UNION",
                 "UNIQUE", "UNKNOWN", "UPDATE", "UPDATETEXT", "USE",
                 "USER", "USING", "VALIDATE", "VALUES", 
                 "VARBINARY", "VARCHAR", "VARIABLE", "VARYING", "VIEW",
                 "WAITFOR", "WHEN", "WHERE", "WHILE", "WITH",
                 "WORK", "WRITETEXT"]

